//<c>   Copyright 1989-2000 by Gerry J. Danen; all rights reserved.

//<t>   Event Scheduler
//
//      Wait until <date> <time>, then exit
//
//      <date> can be in dd-mmm-yy format, or one of the following:
//              TOMORROW
//              MONDAY
//              TUESDAY, etc.
//              NEXT_WORKING_DAY (Monday thru Friday)
//
//      <time> is in hh:mm format (military time, unless am/pm present)

#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_kb.h"
#include"gd_scrn.h"


#define pgVERS_ "V3.10"
#define pgTITL_ "Event Scheduler"
#define pgCOPR_ "1989"


//                      is last char of string s equal to char ch?
#define LASTCH(s,ch)    (s[strlen(s)-1]==ch)


UINT32          sched_hh_mm ;
INT32           hhmmsstt, tics ;
INT16           hh, mm, ss, tt ;

UCHAR           dstring[21], tstring[21], daystring[4] ;


void    do_help( void )
{
    printf( "\n"
        "EVENT SCHEDULER\n\n"
        "Syntax: %s <date> <time>\n\n"
        "<date> can be in dd-mmm-yy format, or one of the following:\n"
        "\tTODAY\n"
        "\tTOMORROW\n"
        "\tMONDAY\n"
        "\tTUESDAY, etc.\n"
        "\tNEXT_WORKING_DAY (Monday thru Friday)\n\n"
        "<time> is in hh:mm format (military time)\n\n"
        "Press ESC to cancel the waiting\n\n",
        get_global_program_name() );
    exit( 1 );
}


void    day_for_date( void )
{
    int         dd, mm, yy ;
    INT32       ymd ;

    ST_CLR( daystring );
    ymd = DateStr2ymd( dstring, DSTYP_DAT_DMY2 );
    if ( ymd != ((INT32) ERROR) )
    {
        DateSplit( ymd, &yy, &mm, &dd );
        WhatDay( ymd, daystring, 0 );
    }
}


void    find_next_day( void )
{
    int         dd, mm, yy ;
    INT32       ymd ;

    ymd = DateStr2ymd( dstring, DSTYP_DAT_DMY2 );
    if ( ymd != ((INT32) ERROR) )
    {
        DateSplit( ymd, &yy, &mm, &dd );
        nextday( &dd, &mm, &yy );
        ymd = ((yy)*10000L) + ((mm)*100L) + (dd);
        FormatDate( ymd, dstring, 1 );
        WhatDay( ymd, daystring, 0 );
    }
}


void    find_day_of_week( void )
{
    UCHAR       dstr[21] ;
    INT16       x ;

    strcpy( dstr, dstring );
    strcpy( dstring, TodaysDate() );
    day_for_date();
    LOOP( x, 0, 7, 1 )
    {
        if ( st_eq( dstr, daystring ) )
            return ;
        find_next_day();
    }
    exit( 99 );
}


void    ck_parms( int argc, char *argv[] )
{
    INT16       ret ;
    int         arg = 1 ;
    char        a[81] ;

    dstring[0] = tstring[0] = EOS ;             // clear strings
    while ( --argc )
    {
        st_cpy( a, argv[arg], 81 );
        _strupr( a );

        if ( st_eq( a, "NEXT_WORKING_DAY" ) )
            strcpy( a, "NWD" );
        else
        if ( st_eq( a, "MONDAY"    ) )
            strcpy( a, "MON" );
        else
        if ( st_eq( a, "TUESDAY"   ) )
            strcpy( a, "TUE" );
        else
        if ( st_eq( a, "WEDNESDAY" ) )
            strcpy( a, "WED" );
        else
        if ( st_eq( a, "THURSDAY"  ) )
            strcpy( a, "THU" );
        else
        if ( st_eq( a, "FRIDAY"    ) )
            strcpy( a, "FRI" );
        else
        if ( st_eq( a, "SATURDAY"  ) )
            strcpy( a, "SAT" );
        else
        if ( st_eq( a, "SUNDAY"    ) )
            strcpy( a, "SUN" );
        else
        if ( st_eq(a, "?") || st_eq(a, "/?") || st_eq(a, "/h") )
            do_help();

        if ( strstr( a, ":" ) || (isdigit( a[0] ) && (LASTCH(a,'M') || LASTCH(a,'P') || LASTCH(a,'A'))) )
        {
            st_cpy( tstring, a, 21 );
            ret = TimeStr2Num( a, 0, &sched_hh_mm );
            if ( ret == ERROR )
                ret = TimeStr2Num( a, 1, &sched_hh_mm );
            if ( ret == ERROR )
                ret = TimeStr2Num( a, 2, &sched_hh_mm );
            if ( ret == ERROR )
            {
                printf( "%s is not a valid time.  Use /H for help!\n", a );
                exit( 1 );
            }
            sched_hh_mm /= ((UINT32) 100) ;
        }
        else
        {
            if ( st_eq( a, "NWD" ) ||
                 st_eq( a, "TOMORROW" ) ||
                 st_eq( a, "TODAY" ) ||
                 st_eq( a, "MON" ) ||
                 st_eq( a, "TUE" ) ||
                 st_eq( a, "WED" ) ||
                 st_eq( a, "THU" ) ||
                 st_eq( a, "FRI" ) ||
                 st_eq( a, "SAT" ) ||
                 st_eq( a, "SUN" ) )
            {
                    st_cpy( dstring, a, 21 );
            }
            else
            {
                INT32 ymd ;
                INT16 dtype ;

                if ( strlen(a) == 7 )
                    dtype = DSTYP_DAT_DMY1 ;    // ddmmmyy
                else
                if ( strlen(a) == 9 )
                    dtype = DSTYP_DAT_DMY2 ;    // dd-mmm-yy
                else
                    dtype = DSTYP_DAT_DMY3 ;    // dd-mmm-yyyy
                ymd = DateStr2ymd( a, dtype );
                if ( ymd == (INT32) ERROR )
                {
                    printf( "%s is not a valid date.  Use /H for help!\n", a );
                    exit( 1 );
                }
                FormatDate( ymd, dstring, 1 );
            }
        }
        arg += 1 ;
    }
}


void    figure_date_n_time( void )
{
    if ( (! strlen( dstring )) && (! strlen( tstring )) )
    {
        strcpy( dstring, "NWD" );
        strcpy( tstring, "00:01A" );
    }
    if ( ! strlen( dstring ) )                  // only time given
    {

//      if tstring > current time, date = today, else date = tomorrow

    }
    if ( ! strlen( tstring ) )                  // only date given
    {
        strcpy( tstring, "00:01A" );
    }
    if ( st_eq( dstring, "TODAY" ) )
        strcpy( dstring, TodaysDate() );
    if ( st_eq( dstring, "TOMORROW" ) )
    {
        strcpy( dstring, TodaysDate() );
        find_next_day();
    }
    day_for_date();

    if ( st_eq( dstring, "NWD" ) )
    {
        if ( st_eq( TodaysDay(), "FRI" ) || st_eq( TodaysDay(), "SAT" ) )
        {
            strcpy( dstring, "MON" );
        }
        else
        {
            strcpy( dstring, TodaysDate() );
            find_next_day();
        }
    }
    if ( st_eq( dstring, "MON" ) || st_eq( dstring, "TUE" ) || st_eq( dstring, "WED" ) ||
         st_eq( dstring, "THU" ) || st_eq( dstring, "FRI" ) || st_eq( dstring, "SAT" ) || st_eq( dstring, "SUN" ) )
    {
        find_day_of_week();
    }
}


void    wait_for_scheduled_date_n_time( void )
{
    UCHAR       prv_time[8] ;
    INT16       ch ;

    printf( "Waiting until %s %s ... press Escape to cancel!\n", dstring, tstring );
    ST_CLR( prv_time );
    for ( ;; )
    {
        if ( ! st_eq( prv_time, TodaysTime() ) )
        {
            printf( "\r\t  %s %s %s", TodaysDay(), TodaysDate(), TodaysTime() );
            st_cpy( prv_time, TodaysTime(), 8 );
            //                                  check date & time
            if ( st_eq( dstring, TodaysDate() ) )
            {
                tm_getsystemtime( &hhmmsstt, &tics, &hh, &mm, &ss, &tt );
                hhmmsstt /= ((INT32) 10000) ;

// printf( "\r  -- %8lu -- %8ld", sched_hh_mm, hhmmsstt );

                if ( (INT32) sched_hh_mm <= hhmmsstt )
                {
                    printf( "\nIt's happening!\n" );
                    exit( 0 );
                }
            }
        }
        ch = kb_getkey_nowait();
        if ( ch == ESCAPE )
        {
            printf ( " ... CANCELLED\n" );
            exit( 3 );
        }
    }
}


//  TM_S2NUM.C  Convert Time from String to UINT32


void    main( int argc, char *argv[] )
{
    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);
    ck_parms( argc, argv );

    figure_date_n_time();
    wait_for_scheduled_date_n_time();

    exit( 0 );
}
